﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.IO;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Wpf;

namespace _9102
{
    public class WicBitmapLayerOverlay : LayerOverlay
    {
        private static readonly int limitThreadCount = Environment.ProcessorCount;

        [NonSerialized]
        private int limitThreadIndex;

        [NonSerialized]
        private Collection<Collection<Layer>> copiedLayers;

        public WicBitmapLayerOverlay()
            : this(new Collection<Layer>())
        { }

        public WicBitmapLayerOverlay(IEnumerable<Layer> layers)
            : base(layers)
        {
            copiedLayers = new Collection<Collection<Layer>>();
            DrawingQuality = DrawingQuality.HighSpeed;
        }

        public DrawingQuality DrawingQuality { get; set; }

        protected override void DrawTileCore(ThinkGeo.MapSuite.Wpf.Tile tile, RectangleShape targetExtent)
        {
            LayerTile layerTile = tile as LayerTile;
            lock (layerTile.DrawingLayers)
            {
                layerTile.DrawingLayers.Clear();
                if (LockLayerMode == LockLayerMode.DoNotLock)
                {
                    int currentThreadCopyIndex = limitThreadIndex++;
                    if (copiedLayers.Count > currentThreadCopyIndex)
                    {
                        Collection<Layer> oneCopies = copiedLayers[currentThreadCopyIndex];

                        foreach (Layer layer in oneCopies)
                        {
                            layerTile.DrawingLayers.Add(layer);
                        }
                    }

                    if (limitThreadIndex >= limitThreadCount || limitThreadIndex >= copiedLayers.Count) limitThreadIndex = 0;
                }
                else
                {
                    foreach (Layer layer in Layers)
                    {
                        layerTile.DrawingLayers.Add(layer);
                    }
                }
            }

            int tw = (int)tile.Width;
            int th = (int)tile.Height;

            GeoCanvas geoCanvas = null;
            object nativeImage = null;

            if (TileCache != null) { layerTile.TileCache = TileCache; }
            nativeImage = new Bitmap(tw, th);

            geoCanvas = new WicBitmapGeoCanvas
            {
                DrawingQuality = DrawingQuality,
            };

            geoCanvas.BeginDrawing(nativeImage, targetExtent, MapArguments.MapUnit);
            if (tile.IsAsync)
            {
                layerTile.DrawAsync(geoCanvas);
            }
            else
            {
                layerTile.Draw(geoCanvas);
                geoCanvas.EndDrawing();
                layerTile.CommitDrawing(geoCanvas, GetImageSourceFromNativeImage(nativeImage));
            }
        }

        private static object GetImageSourceFromNativeImage(object nativeImage)
        {
            object imageSource = nativeImage;
            if (nativeImage is System.Drawing.Bitmap)
            {
                System.Drawing.Bitmap bitmap = (System.Drawing.Bitmap)nativeImage;
                MemoryStream memoryStream = new MemoryStream();
                bitmap.Save(memoryStream, System.Drawing.Imaging.ImageFormat.Png);
                memoryStream.Seek(0, SeekOrigin.Begin);
                imageSource = memoryStream;
                bitmap.Dispose();
            }

            return imageSource;
        }
    }
}
